@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2012 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -776,7 +776,7 @@ static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_demux_stream_t *str
       lprintf("packet done: offset=%d, payload=%d\n", stream->frag_offset, stream->payload_size);
     }
 
-    lprintf ("buffer type %08x %8d bytes, %8" PRId64 " pts\n",
+    lprintf ("buffer type %08x %8d bytes, %8lld pts\n",
              buf->type, buf->size, buf->pts);
 
     stream->fifo->put (stream->fifo, buf);
@@ -857,7 +857,7 @@ static void asf_send_buffer_defrag (demux_asf_t *this, asf_demux_stream_t *strea
       buf->type       = stream->buf_type;
       buf->size       = bufsize;
 
-      lprintf ("buffer type %08x %8d bytes, %8" PRId64 " pts\n",
+      lprintf ("buffer type %08x %8d bytes, %8lld pts\n",
 	       buf->type, buf->size, buf->pts);
 
       stream->frag_offset -= bufsize;
@@ -929,11 +929,8 @@ static int asf_parse_packet_align(demux_asf_t *this) {
 static int asf_parse_packet_ecd(demux_asf_t *this, uint32_t  *p_hdr_size) {
 
   uint8_t   ecd_flags;
+  uint8_t   buf[16];
   int       invalid_packet;
-  union {
-    uint8_t  u8[16];
-    GUID     guid;
-  } buf;
 
   do {
     ecd_flags = get_byte(this); *p_hdr_size = 1;
@@ -955,7 +952,7 @@ static int asf_parse_packet_ecd(demux_asf_t *this, uint32_t  *p_hdr_size) {
       if (ecd_present && !ecd_opaque && !ecd_len_type) {
         int read_size;
 
-        read_size = this->input->read (this->input, buf.u8, ecd_len);
+        read_size = this->input->read (this->input, buf, ecd_len);
         if (read_size != ecd_len) {
           this->status = DEMUX_FINISHED;
           return 1;
@@ -963,18 +960,19 @@ static int asf_parse_packet_ecd(demux_asf_t *this, uint32_t  *p_hdr_size) {
         *p_hdr_size += read_size;
 
       } else {
+        GUID *guid = (GUID *)buf;
 
         /* check if it's a new stream */
-        buf.u8[0] = ecd_flags;
-        if (this->input->read (this->input, buf.u8 + 1, 15) != 15) {
+        buf[0] = ecd_flags;
+        if (this->input->read (this->input, buf + 1, 15) != 15) {
           this->status = DEMUX_FINISHED;
           return 1;
         }
         *p_hdr_size += 15;
-        buf.guid.Data1 = _X_LE_32(buf.u8);
-        buf.guid.Data2 = _X_LE_16(buf.u8 + 4);
-        buf.guid.Data3 = _X_LE_16(buf.u8 + 6);
-        if (get_guid_id(this, &buf.guid) == GUID_ASF_HEADER) {
+        guid->Data1 = _X_LE_32(buf);
+        guid->Data2 = _X_LE_16(buf + 4);
+        guid->Data3 = _X_LE_16(buf + 6);
+        if (get_guid_id(this, guid) == GUID_ASF_HEADER) {
           lprintf("new asf header detected\n");
           _x_demux_control_end(this->stream, 0);
           if (demux_asf_send_headers_common(this))
@@ -1041,14 +1039,14 @@ static int asf_parse_packet_payload_header(demux_asf_t *this, uint32_t p_hdr_siz
 #ifdef LOG
   timestamp = get_le32(this); p_hdr_size += 4;
   duration  = get_le16(this); p_hdr_size += 2;
-
-  lprintf ("timestamp=%"PRId64", duration=%"PRId64"\n", timestamp, duration);
 #else
   /* skip the above bytes */
   this->input->seek (this->input, 6, SEEK_CUR);
   p_hdr_size += 6;
 #endif
 
+  lprintf ("timestamp=%"PRId64", duration=%"PRId64"\n", timestamp, duration);
+
   if ((this->packet_len_flags >> 5) & 3) {
     /* absolute data size */
     lprintf ("absolute data size\n");
