@@ -33,80 +33,78 @@
 #include "cpb.h"
 
 /* default scaling_lists according to Table 7-2 */
-static const uint8_t default_4x4_intra[16] = { 6, 13, 13, 20, 20, 20, 28, 28, 28, 28, 32,
+uint8_t default_4x4_intra[16] = { 6, 13, 13, 20, 20, 20, 28, 28, 28, 28, 32,
     32, 32, 37, 37, 42 };
 
-static const uint8_t default_4x4_inter[16] = { 10, 14, 14, 20, 20, 20, 24, 24, 24, 24, 27,
+uint8_t default_4x4_inter[16] = { 10, 14, 14, 20, 20, 20, 24, 24, 24, 24, 27,
     27, 27, 30, 30, 34 };
 
-static const uint8_t default_8x8_intra[64] = { 6, 10, 10, 13, 11, 13, 16, 16, 16, 16, 18,
+uint8_t default_8x8_intra[64] = { 6, 10, 10, 13, 11, 13, 16, 16, 16, 16, 18,
     18, 18, 18, 18, 23, 23, 23, 23, 23, 23, 25, 25, 25, 25, 25, 25, 25, 27, 27,
     27, 27, 27, 27, 27, 27, 29, 29, 29, 29, 29, 29, 29, 31, 31, 31, 31, 31, 31,
     33, 33, 33, 33, 33, 36, 36, 36, 36, 38, 38, 38, 40, 40, 42 };
 
-static const uint8_t default_8x8_inter[64] = { 9, 13, 13, 15, 13, 15, 17, 17, 17, 17, 19,
+uint8_t default_8x8_inter[64] = { 9, 13, 13, 15, 13, 15, 17, 17, 17, 17, 19,
     19, 19, 19, 19, 21, 21, 21, 21, 21, 21, 22, 22, 22, 22, 22, 22, 22, 24, 24,
     24, 24, 24, 24, 24, 24, 25, 25, 25, 25, 25, 25, 25, 27, 27, 27, 27, 27, 27,
     28, 28, 28, 28, 28, 30, 30, 30, 30, 32, 32, 32, 33, 33, 35 };
 
 struct buf_reader
 {
-  const uint8_t *buf;
-  const uint8_t *cur_pos;
+  uint8_t *buf;
+  uint8_t *cur_pos;
   int len;
   int cur_offset;
 };
 
-static struct h264_parser* init_parser();
+struct h264_parser* init_parser();
 
 static inline uint32_t read_bits(struct buf_reader *buf, int len);
-static uint32_t read_exp_golomb(struct buf_reader *buf);
-static int32_t read_exp_golomb_s(struct buf_reader *buf);
+uint32_t read_exp_golomb(struct buf_reader *buf);
+int32_t read_exp_golomb_s(struct buf_reader *buf);
 
-static void calculate_pic_order(struct h264_parser *parser, struct coded_picture *pic,
+void calculate_pic_order(struct h264_parser *parser, struct coded_picture *pic,
     struct slice_header *slc);
-#if 0
-static void skip_scaling_list(struct buf_reader *buf, int size);
-#endif
-static void parse_scaling_list(struct buf_reader *buf, uint8_t *scaling_list,
+void skip_scaling_list(struct buf_reader *buf, int size);
+void parse_scaling_list(struct buf_reader *buf, uint8_t *scaling_list,
     int length, int index);
 
-static struct nal_unit* parse_nal_header(struct buf_reader *buf,
+struct nal_unit* parse_nal_header(struct buf_reader *buf,
     struct coded_picture *pic, struct h264_parser *parser);
 static void sps_scaling_list_fallback(struct seq_parameter_set_rbsp *sps,
     int i);
 static void pps_scaling_list_fallback(struct seq_parameter_set_rbsp *sps,
     struct pic_parameter_set_rbsp *pps, int i);
 
-static uint8_t parse_sps(struct buf_reader *buf, struct seq_parameter_set_rbsp *sps);
-static void interpret_sps(struct coded_picture *pic, struct h264_parser *parser);
+uint8_t parse_sps(struct buf_reader *buf, struct seq_parameter_set_rbsp *sps);
+void interpret_sps(struct coded_picture *pic, struct h264_parser *parser);
 
-static void parse_vui_parameters(struct buf_reader *buf,
+void parse_vui_parameters(struct buf_reader *buf,
     struct seq_parameter_set_rbsp *sps);
-static void parse_hrd_parameters(struct buf_reader *buf, struct hrd_parameters *hrd);
+void parse_hrd_parameters(struct buf_reader *buf, struct hrd_parameters *hrd);
 
-static uint8_t parse_pps(struct buf_reader *buf, struct pic_parameter_set_rbsp *pps);
-static void interpret_pps(struct coded_picture *pic);
+uint8_t parse_pps(struct buf_reader *buf, struct pic_parameter_set_rbsp *pps);
+void interpret_pps(struct coded_picture *pic);
 
-static void parse_sei(struct buf_reader *buf, struct sei_message *sei,
+void parse_sei(struct buf_reader *buf, struct sei_message *sei,
     struct h264_parser *parser);
-static void interpret_sei(struct coded_picture *pic);
+void interpret_sei(struct coded_picture *pic);
 
-static uint8_t parse_slice_header(struct buf_reader *buf, struct nal_unit *slc_nal,
+uint8_t parse_slice_header(struct buf_reader *buf, struct nal_unit *slc_nal,
     struct h264_parser *parser);
-static void interpret_slice_header(struct h264_parser *parser, struct nal_unit *slc_nal);
+void interpret_slice_header(struct h264_parser *parser, struct nal_unit *slc_nal);
 
-static void parse_ref_pic_list_reordering(struct buf_reader *buf,
+void parse_ref_pic_list_reordering(struct buf_reader *buf,
     struct slice_header *slc);
 
-static void calculate_pic_nums(struct h264_parser *parser, struct coded_picture *cpic);
-static void execute_ref_pic_marking(struct coded_picture *cpic,
+void calculate_pic_nums(struct h264_parser *parser, struct coded_picture *cpic);
+void execute_ref_pic_marking(struct coded_picture *cpic,
     uint32_t memory_management_control_operation,
     uint32_t marking_nr,
     struct h264_parser *parser);
-static void parse_pred_weight_table(struct buf_reader *buf, struct slice_header *slc,
+void parse_pred_weight_table(struct buf_reader *buf, struct slice_header *slc,
     struct h264_parser *parser);
-static void parse_dec_ref_pic_marking(struct buf_reader *buf,
+void parse_dec_ref_pic_marking(struct buf_reader *buf,
     struct nal_unit *slc_nal);
 
 /* here goes the parser implementation */
@@ -212,9 +210,9 @@ static inline uint32_t read_bits(struct buf_reader *buf, int len)
 }
 
 /* determines if following bits are rtsb_trailing_bits */
-static inline int rbsp_trailing_bits(const uint8_t *buf, int buf_len)
+static inline int rbsp_trailing_bits(uint8_t *buf, int buf_len)
 {
-  const uint8_t *cur_buf = buf+(buf_len-1);
+  uint8_t *cur_buf = buf+(buf_len-1);
   uint8_t cur_val;
   int parsed_bits = 0;
   int i;
@@ -234,19 +232,19 @@ static inline int rbsp_trailing_bits(const uint8_t *buf, int buf_len)
   return 0;
 }
 
-static uint32_t read_exp_golomb(struct buf_reader *buf)
+uint32_t read_exp_golomb(struct buf_reader *buf)
 {
   int leading_zero_bits = 0;
 
   while (read_bits(buf, 1) == 0 && leading_zero_bits < 32)
     leading_zero_bits++;
 
-  uint32_t code = ((uint64_t)1 << leading_zero_bits) - 1 + read_bits(buf,
+  uint32_t code = (1 << leading_zero_bits) - 1 + read_bits(buf,
       leading_zero_bits);
   return code;
 }
 
-static int32_t read_exp_golomb_s(struct buf_reader *buf)
+int32_t read_exp_golomb_s(struct buf_reader *buf)
 {
   uint32_t ue = read_exp_golomb(buf);
   int32_t code = ue & 0x01 ? (ue + 1) / 2 : -(ue / 2);
@@ -258,7 +256,7 @@ static int32_t read_exp_golomb_s(struct buf_reader *buf)
  * parses the NAL header data and calls the subsequent
  * parser methods that handle specific NAL units
  */
-static struct nal_unit* parse_nal_header(struct buf_reader *buf,
+struct nal_unit* parse_nal_header(struct buf_reader *buf,
     struct coded_picture *pic, struct h264_parser *parser)
 {
   if (buf->len < 1)
@@ -305,7 +303,7 @@ static struct nal_unit* parse_nal_header(struct buf_reader *buf,
  * calculates the picture order count according to ITU-T Rec. H.264 (11/2007)
  * chapter 8.2.1, p104f
  */
-static void calculate_pic_order(struct h264_parser *parser, struct coded_picture *pic,
+void calculate_pic_order(struct h264_parser *parser, struct coded_picture *pic,
     struct slice_header *slc)
 {
   /* retrieve sps and pps from the buffers */
@@ -410,17 +408,15 @@ static void calculate_pic_order(struct h264_parser *parser, struct coded_picture
   }
 }
 
-#if 0
-static void skip_scaling_list(struct buf_reader *buf, int size)
+void skip_scaling_list(struct buf_reader *buf, int size)
 {
   int i;
   for (i = 0; i < size; i++) {
     read_exp_golomb_s(buf);
   }
 }
-#endif
 
-static void parse_scaling_list(struct buf_reader *buf, uint8_t *scaling_list,
+void parse_scaling_list(struct buf_reader *buf, uint8_t *scaling_list,
     int length, int index)
 {
   int last_scale = 8;
@@ -545,7 +541,7 @@ static void pps_scaling_list_fallback(struct seq_parameter_set_rbsp *sps, struct
 }
 
 
-static uint8_t parse_sps(struct buf_reader *buf, struct seq_parameter_set_rbsp *sps)
+uint8_t parse_sps(struct buf_reader *buf, struct seq_parameter_set_rbsp *sps)
 {
   sps->profile_idc = read_bits(buf, 8);
   sps->constraint_setN_flag = read_bits(buf, 4);
@@ -640,7 +636,7 @@ static uint8_t parse_sps(struct buf_reader *buf, struct seq_parameter_set_rbsp *
 /* evaluates values parsed by sps and modifies the current
  * picture according to them
  */
-static void interpret_sps(struct coded_picture *pic, struct h264_parser *parser)
+void interpret_sps(struct coded_picture *pic, struct h264_parser *parser)
 {
   if(pic->sps_nal == NULL) {
     xprintf(parser->xine, XINE_VERBOSITY_DEBUG,
@@ -677,7 +673,7 @@ static void interpret_sps(struct coded_picture *pic, struct h264_parser *parser)
   }
 }
 
-static void parse_sei(struct buf_reader *buf, struct sei_message *sei,
+void parse_sei(struct buf_reader *buf, struct sei_message *sei,
     struct h264_parser *parser)
 {
   uint8_t tmp;
@@ -779,7 +775,7 @@ static void parse_sei(struct buf_reader *buf, struct sei_message *sei,
 
 }
 
-static void interpret_sei(struct coded_picture *pic)
+void interpret_sei(struct coded_picture *pic)
 {
   if(!pic->sps_nal || !pic->sei_nal)
     return;
@@ -816,7 +812,7 @@ static void interpret_sei(struct coded_picture *pic)
   }
 }
 
-static void parse_vui_parameters(struct buf_reader *buf,
+void parse_vui_parameters(struct buf_reader *buf,
     struct seq_parameter_set_rbsp *sps)
 {
   sps->vui_parameters.aspect_ration_info_present_flag = read_bits(buf, 1);
@@ -887,7 +883,7 @@ static void parse_vui_parameters(struct buf_reader *buf,
   }
 }
 
-static void parse_hrd_parameters(struct buf_reader *buf, struct hrd_parameters *hrd)
+void parse_hrd_parameters(struct buf_reader *buf, struct hrd_parameters *hrd)
 {
   hrd->cpb_cnt_minus1 = read_exp_golomb(buf);
   hrd->bit_rate_scale = read_bits(buf, 4);
@@ -906,7 +902,7 @@ static void parse_hrd_parameters(struct buf_reader *buf, struct hrd_parameters *
   hrd->time_offset_length = read_bits(buf, 5);
 }
 
-static uint8_t parse_pps(struct buf_reader *buf, struct pic_parameter_set_rbsp *pps)
+uint8_t parse_pps(struct buf_reader *buf, struct pic_parameter_set_rbsp *pps)
 {
   pps->pic_parameter_set_id = read_exp_golomb(buf);
   pps->seq_parameter_set_id = read_exp_golomb(buf);
@@ -985,7 +981,7 @@ static uint8_t parse_pps(struct buf_reader *buf, struct pic_parameter_set_rbsp *
   return 0;
 }
 
-static void interpret_pps(struct coded_picture *pic)
+void interpret_pps(struct coded_picture *pic)
 {
   if(pic->sps_nal == NULL) {
     lprintf("WARNING: Picture contains no seq_parameter_set\n");
@@ -1013,7 +1009,7 @@ static void interpret_pps(struct coded_picture *pic)
   }
 }
 
-static uint8_t parse_slice_header(struct buf_reader *buf, struct nal_unit *slc_nal,
+uint8_t parse_slice_header(struct buf_reader *buf, struct nal_unit *slc_nal,
     struct h264_parser *parser)
 {
   struct slice_header *slc = &slc_nal->slc;
@@ -1125,7 +1121,7 @@ static uint8_t parse_slice_header(struct buf_reader *buf, struct nal_unit *slc_n
   return 0;
 }
 
-static void interpret_slice_header(struct h264_parser *parser, struct nal_unit *slc_nal)
+void interpret_slice_header(struct h264_parser *parser, struct nal_unit *slc_nal)
 {
   struct coded_picture *pic = parser->pic;
   struct slice_header *slc = &slc_nal->slc;
@@ -1163,7 +1159,7 @@ static void interpret_slice_header(struct h264_parser *parser, struct nal_unit *
   pic->pps_nal = pps_nal;
 }
 
-static void parse_ref_pic_list_reordering(struct buf_reader *buf, struct slice_header *slc)
+void parse_ref_pic_list_reordering(struct buf_reader *buf, struct slice_header *slc)
 {
   if (slc->slice_type != SLICE_I && slc->slice_type != SLICE_SI) {
     slc->ref_pic_list_reordering.ref_pic_list_reordering_flag_l0 = read_bits(
@@ -1208,7 +1204,7 @@ static void parse_ref_pic_list_reordering(struct buf_reader *buf, struct slice_h
   }
 }
 
-static void parse_pred_weight_table(struct buf_reader *buf, struct slice_header *slc,
+void parse_pred_weight_table(struct buf_reader *buf, struct slice_header *slc,
     struct h264_parser *parser)
 {
   /* retrieve sps and pps from the buffers */
@@ -1286,7 +1282,7 @@ static void parse_pred_weight_table(struct buf_reader *buf, struct slice_header
  * PicNum calculation following ITU-T H264 11/2007
  * 8.2.4.1 p112f
  */
-static void calculate_pic_nums(struct h264_parser *parser, struct coded_picture *cpic)
+void calculate_pic_nums(struct h264_parser *parser, struct coded_picture *cpic)
 {
   struct decoded_picture *pic = NULL;
   struct slice_header *cslc = &cpic->slc_nal->slc;
@@ -1334,7 +1330,7 @@ static void calculate_pic_nums(struct h264_parser *parser, struct coded_picture
   }
 }
 
-static void execute_ref_pic_marking(struct coded_picture *cpic,
+void execute_ref_pic_marking(struct coded_picture *cpic,
     uint32_t memory_management_control_operation,
     uint32_t marking_nr,
     struct h264_parser *parser)
@@ -1499,7 +1495,7 @@ static void execute_ref_pic_marking(struct coded_picture *cpic,
   }
 }
 
-static void parse_dec_ref_pic_marking(struct buf_reader *buf,
+void parse_dec_ref_pic_marking(struct buf_reader *buf,
     struct nal_unit *slc_nal)
 {
   struct slice_header *slc = &slc_nal->slc;
@@ -1556,7 +1552,7 @@ static void parse_dec_ref_pic_marking(struct buf_reader *buf,
 
 /* ----------------- NAL parser ----------------- */
 
-static struct h264_parser* init_parser(xine_t *xine)
+struct h264_parser* init_parser(xine_t *xine)
 {
   struct h264_parser *parser = calloc(1, sizeof(struct h264_parser));
   parser->pic = create_coded_picture();
@@ -1570,8 +1566,7 @@ static struct h264_parser* init_parser(xine_t *xine)
   return parser;
 }
 
-#if 0
-static void reset_parser(struct h264_parser *parser)
+void reset_parser(struct h264_parser *parser)
 {
   parser->position = NON_VCL;
   parser->buf_len = parser->prebuf_len = 0;
@@ -1595,9 +1590,8 @@ static void reset_parser(struct h264_parser *parser)
     parser->pic = create_coded_picture();
   }
 }
-#endif
 
-static void free_parser(struct h264_parser *parser)
+void free_parser(struct h264_parser *parser)
 {
   dpb_free_all(parser->dpb);
   release_dpb(parser->dpb);
@@ -1606,7 +1600,7 @@ static void free_parser(struct h264_parser *parser)
   free(parser);
 }
 
-static void parse_codec_private(struct h264_parser *parser, const uint8_t *inbuf, int inbuf_len)
+void parse_codec_private(struct h264_parser *parser, uint8_t *inbuf, int inbuf_len)
 {
   struct buf_reader bufr;
 
@@ -1664,7 +1658,7 @@ static void parse_codec_private(struct h264_parser *parser, const uint8_t *inbuf
   nal_buffer_append(parser->sps_buffer, nal);
 }
 
-static void process_mmc_operations(struct h264_parser *parser, struct coded_picture *picture)
+void process_mmc_operations(struct h264_parser *parser, struct coded_picture *picture)
 {
   if (picture->flag_mask & REFERENCE) {
     parser->prev_pic_order_cnt_lsb
@@ -1683,9 +1677,9 @@ static void process_mmc_operations(struct h264_parser *parser, struct coded_pict
   }
 }
 
-static int parse_frame(struct h264_parser *parser, const uint8_t *inbuf, int inbuf_len,
+int parse_frame(struct h264_parser *parser, uint8_t *inbuf, int inbuf_len,
     int64_t pts,
-    const void **ret_buf, uint32_t *ret_len, struct coded_picture **ret_pic)
+    uint8_t **ret_buf, uint32_t *ret_len, struct coded_picture **ret_pic)
 {
   int32_t next_nal = 0;
   int32_t offset = 0;
@@ -1734,11 +1728,9 @@ static int parse_frame(struct h264_parser *parser, const uint8_t *inbuf, int inb
 
       //lprintf("Frame complete: %d bytes\n", parser->buf_len);
       *ret_len = parser->buf_len;
-      {
-        uint8_t *rbuf = malloc(parser->buf_len);
-        *ret_buf = rbuf;
-        if (rbuf) xine_fast_memcpy(rbuf, parser->buf, parser->buf_len);
-      }
+      *ret_buf = malloc(parser->buf_len);
+      xine_fast_memcpy(*ret_buf, parser->buf, parser->buf_len);
+
       *ret_pic = completed_pic;
 
       parser->buf_len = 0;
@@ -1815,7 +1807,7 @@ static int parse_frame(struct h264_parser *parser, const uint8_t *inbuf, int inb
  *         1: NAL is the beginning of a new coded picture
  *         3: NAL is marked as END_OF_SEQUENCE
  */
-static int parse_nal(const uint8_t *buf, int buf_len, struct h264_parser *parser,
+int parse_nal(uint8_t *buf, int buf_len, struct h264_parser *parser,
     struct coded_picture **completed_picture)
 {
   int ret = 0;
@@ -2000,7 +1992,7 @@ static int parse_nal(const uint8_t *buf, int buf_len, struct h264_parser *parser
   return ret;
 }
 
-static int seek_for_nal(uint8_t *buf, int buf_len, struct h264_parser *parser)
+int seek_for_nal(uint8_t *buf, int buf_len, struct h264_parser *parser)
 {
   if(buf_len <= 0)
     return -1;
