@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2011 the xine project
  * May 2003 - Miguel Freitas
  * This feature was sponsored by 1Control
  *
@@ -49,23 +49,13 @@
 #include <signal.h>
 #include <errno.h>
 #include <sys/time.h>
-#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
-#endif
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
-#ifdef HAVE_SYS_IOCTL_H
+
+#ifndef WIN32
 #include <sys/ioctl.h>
-#endif
-#ifdef WIN32
-#include <ws2tcpip.h>  // socklen_t
+#include <arpa/inet.h>
+#include <netdb.h>
 #endif
 
 #include <dlfcn.h>
@@ -123,10 +113,10 @@ static int sock_check_opened(int socket) {
 /*
  * Write to socket.
  */
-static int sock_data_write(xine_t *xine, int socket, const void *buf_gen, int len) {
+static int sock_data_write(xine_t *xine, int socket, void *buf_gen, int len) {
   ssize_t  size;
   int      wlen = 0;
-  const uint8_t *buf = buf_gen;
+  uint8_t *buf = buf_gen;
 
   if((socket < 0) || (buf == NULL))
     return -1;
@@ -170,23 +160,24 @@ sock_string_write(xine_t *xine, int socket, const char *msg, ...) {
  * this is the most important broadcaster function.
  * it sends data to every connected client (slaves).
  */
-static void broadcaster_data_write(broadcaster_t *this, const void *buf, int len) {
+static void broadcaster_data_write(broadcaster_t *this, void *buf, int len) {
   xine_list_iterator_t ite;
-  int ssock;
 
-  ite = NULL;
-  ssock = (intptr_t)xine_list_next_value (this->connections, &ite);
+  ite = xine_list_front (this->connections);
   while (ite) {
+
+    int *psock = xine_list_get_value(this->connections, ite);
+
+    ite = xine_list_next(this->connections, ite);
+
     /* in case of failure remove from list */
-    if (sock_data_write (this->stream->xine, ssock, buf, len) < 0) {
-      xine_list_iterator_t failed;
-      xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "broadcaster: closing socket %d\n", ssock);
-      close (ssock);
-      failed = ite;
-      ssock = (intptr_t)xine_list_next_value (this->connections, &ite);
-      xine_list_remove (this->connections, failed);
-    } else {
-      ssock = (intptr_t)xine_list_next_value (this->connections, &ite);
+    if( sock_data_write(this->stream->xine, *psock, buf, len) < 0 ) {
+
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "broadcaster: closing socket %d\n", *psock);
+      close(*psock);
+      free(psock);
+
+      xine_list_remove (this->connections, xine_list_prev(this->connections, ite));
     }
   }
 }
@@ -242,9 +233,12 @@ static void *manager_loop (void *this_gen) {
 
           /* identification string, helps demuxer probing */
           if( sock_string_write(this->stream->xine, ssock,"master xine v1") > 0 ) {
-            xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-              "broadcaster: new connection socket %d\n", ssock);
-            xine_list_push_back (this->connections, (void *)(intptr_t)ssock);
+            int *psock = malloc(sizeof(int));
+            *psock = ssock;
+
+            xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+		    "broadcaster: new connection socket %d\n", *psock);
+            xine_list_push_back(this->connections, psock);
           }
         }
       }
@@ -295,7 +289,6 @@ static void send_buf (broadcaster_t *this, const char *from, buf_element_t *buf)
 static void video_put_cb (fifo_buffer_t *fifo, buf_element_t *buf, void *this_gen) {
   broadcaster_t *this = (broadcaster_t *) this_gen;
 
-  (void)fifo;
   pthread_mutex_lock( &this->lock );
   send_buf(this, "video", buf);
   pthread_mutex_unlock( &this->lock );
@@ -304,7 +297,6 @@ static void video_put_cb (fifo_buffer_t *fifo, buf_element_t *buf, void *this_ge
 static void audio_put_cb (fifo_buffer_t *fifo, buf_element_t *buf, void *this_gen) {
   broadcaster_t *this = (broadcaster_t *) this_gen;
 
-  (void)fifo;
   pthread_mutex_lock( &this->lock );
   send_buf(this, "audio", buf);
   pthread_mutex_unlock( &this->lock );
@@ -332,25 +324,14 @@ broadcaster_t *_x_init_broadcaster(xine_stream_t *stream, int port)
   if(bind(msock, &servAddr.sa, sizeof(servAddr.in))<0)
   {
     xprintf(stream->xine, XINE_VERBOSITY_DEBUG, "broadcaster: error binding to port %d\n", port);
-    close(msock);
     return NULL;
   }
 
-  if (listen(msock,QLEN) < 0) {
-    xprintf(stream->xine, XINE_VERBOSITY_DEBUG, "broadcaster: error listening port %d\n", port);
-    close(msock);
-    return NULL;
-  }
+  listen(msock,QLEN);
 
-#ifndef _WIN32
   signal( SIGPIPE, SIG_IGN );
-#endif
-  this = calloc(1, sizeof(broadcaster_t));
-  if (!this) {
-    close(msock);
-    return NULL;
-  }
 
+  this = calloc(1, sizeof(broadcaster_t));
   this->port = port;
   this->stream = stream;
   this->msock = msock;
@@ -358,8 +339,7 @@ broadcaster_t *_x_init_broadcaster(xine_stream_t *stream, int port)
 
   pthread_mutex_init (&this->lock, NULL);
 
-  if (stream->video_fifo)
-    stream->video_fifo->register_put_cb(stream->video_fifo, video_put_cb, this);
+  stream->video_fifo->register_put_cb(stream->video_fifo, video_put_cb, this);
 
   if(stream->audio_fifo)
     stream->audio_fifo->register_put_cb(stream->audio_fifo, audio_put_cb, this);
@@ -369,48 +349,42 @@ broadcaster_t *_x_init_broadcaster(xine_stream_t *stream, int port)
                              NULL, manager_loop, (void *)this)) != 0) {
     xprintf (stream->xine, XINE_VERBOSITY_NONE,
 	     "broadcaster: can't create new thread (%s)\n", strerror(err));
-
-    this->running = 0;
-    _x_close_broadcaster(this);
-    return NULL;
+    _x_abort();
   }
 
   return this;
 }
 
-void _x_close_broadcaster(broadcaster_t *this_gen)
+void _x_close_broadcaster(broadcaster_t *this)
 {
-  if (this_gen->running) {
-    this_gen->running = 0;
-    pthread_cancel(this_gen->manager_thread);
-    pthread_join(this_gen->manager_thread,NULL);
+  this->running = 0;
+  pthread_cancel(this->manager_thread);
+  pthread_join(this->manager_thread,NULL);
+  close(this->msock);
+
+  if (this->stream->video_fifo)
+    this->stream->video_fifo->unregister_put_cb(this->stream->video_fifo, video_put_cb);
+
+  if(this->stream->audio_fifo)
+    this->stream->audio_fifo->unregister_put_cb(this->stream->audio_fifo, audio_put_cb);
+
+  xine_list_iterator_t ite;
+
+  while ( (ite = xine_list_front(this->connections)) ) {
+    int *psock = xine_list_get_value(this->connections, ite);
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "broadcaster: closing socket %d\n", *psock);
+    close(*psock);
+    free(psock);
+    xine_list_remove (this->connections, ite);
   }
-  close(this_gen->msock);
+  xine_list_delete(this->connections);
 
-  if (this_gen->stream->video_fifo)
-    this_gen->stream->video_fifo->unregister_put_cb(this_gen->stream->video_fifo, video_put_cb);
+  pthread_mutex_destroy( &this->lock );
 
-  if(this_gen->stream->audio_fifo)
-    this_gen->stream->audio_fifo->unregister_put_cb(this_gen->stream->audio_fifo, audio_put_cb);
-
-  xine_list_iterator_t ite = NULL;
-
-  while (1) {
-    int ssock = (intptr_t)xine_list_next_value (this_gen->connections, &ite);
-    if (!ite)
-      break;
-    xprintf (this_gen->stream->xine, XINE_VERBOSITY_DEBUG, "broadcaster: closing socket %d\n", ssock);
-    close (ssock);
-  }
-  xine_list_delete(this_gen->connections);
-
-  pthread_mutex_destroy( &this_gen->lock );
-
-  free(this_gen);
+  free(this);
 }
 
-int _x_get_broadcaster_port(broadcaster_t *this_gen)
+int _x_get_broadcaster_port(broadcaster_t *this)
 {
-  return this_gen->port;
+  return this->port;
 }
-
