@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2011 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -23,16 +23,17 @@
 #endif
 
 #include <stdlib.h>
-#include <string.h>
 #include <unistd.h>
-#include <pthread.h>
+#include <string.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 
 #include <xine/buffer.h>
 #include <xine/xine_internal.h>
+#include "bswap.h"
 #include <xine/xineutils.h>
 #include <xine/video_overlay.h>
-#include "bswap.h"
 
 /*
 #define LOG_DEBUG
@@ -123,8 +124,10 @@ static void remove_events_handle( video_overlay_t *this, int32_t handle, int loc
 
       /* free its overlay */
       if( this->events[this_event].event->object.overlay ) {
-        _x_freep( &this->events[this_event].event->object.overlay->rle );
-        _x_freep( &this->events[this_event].event->object.overlay );
+        if( this->events[this_event].event->object.overlay->rle )
+          free( this->events[this_event].event->object.overlay->rle );
+        free(this->events[this_event].event->object.overlay);
+        this->events[this_event].event->object.overlay = NULL;
       }
 
       /* mark as free */
@@ -173,8 +176,10 @@ static void internal_video_overlay_free_handle(video_overlay_t *this, int32_t ha
   if( this->objects[handle].overlay ) {
     set_argb_layer_ptr(&this->objects[handle].overlay->argb_layer, NULL);
 
-    _x_freep( &this->objects[handle].overlay->rle );
-    _x_freep( &this->objects[handle].overlay );
+    if( this->objects[handle].overlay->rle )
+      free( this->objects[handle].overlay->rle );
+    free( this->objects[handle].overlay );
+    this->objects[handle].overlay = NULL;
   }
   this->objects[handle].handle = -1;
 
@@ -213,7 +218,6 @@ static void video_overlay_reset (video_overlay_t *this) {
   }
   pthread_mutex_unlock (&this->events_mutex);
 
-  pthread_mutex_lock( &this->showing_mutex );
   for( i = 0; i < MAX_SHOWING; i++ )
     this->showing[i].handle = -1;
 
@@ -222,7 +226,6 @@ static void video_overlay_reset (video_overlay_t *this) {
   }
 
   this->showing_changed = 0;
-  pthread_mutex_unlock( &this->showing_mutex );
 }
 
 
@@ -397,8 +400,10 @@ static int video_overlay_event( video_overlay_t *this, int64_t vpts ) {
         if (this->events[this_event].event->object.overlay != NULL) {
           set_argb_layer_ptr(&this->events[this_event].event->object.overlay->argb_layer, NULL);
 
-          _x_freep( &this->events[this_event].event->object.overlay->rle );
-          _x_freep( &this->events[this_event].event->object.overlay );
+          if( this->events[this_event].event->object.overlay->rle != NULL )
+            free( this->events[this_event].event->object.overlay->rle );
+          free(this->events[this_event].event->object.overlay);
+          this->events[this_event].event->object.overlay = NULL;
         }
         remove_showing_handle( this, handle );
         break;
@@ -411,8 +416,10 @@ static int video_overlay_event( video_overlay_t *this, int64_t vpts ) {
         if( this->events[this_event].event->object.overlay != NULL) {
           set_argb_layer_ptr(&this->events[this_event].event->object.overlay->argb_layer, NULL);
 
-          _x_freep( &this->events[this_event].event->object.overlay->rle );
-          _x_freep( &this->events[this_event].event->object.overlay );
+          if( this->events[this_event].event->object.overlay->rle != NULL )
+            free( this->events[this_event].event->object.overlay->rle );
+          free(this->events[this_event].event->object.overlay);
+          this->events[this_event].event->object.overlay = NULL;
         }
         /* this avoid removing this_event from the queue
          * (it will be removed at the end of this loop) */
@@ -473,9 +480,14 @@ static int video_overlay_event( video_overlay_t *this, int64_t vpts ) {
 
         if( this->events[this_event].event->object.overlay->rle ) {
           xprintf (this->xine, XINE_VERBOSITY_DEBUG, "video_overlay: warning EVENT_MENU_BUTTON with rle data\n");
-          _x_freep( &this->events[this_event].event->object.overlay->rle );
+          free( this->events[this_event].event->object.overlay->rle );
+          this->events[this_event].event->object.overlay->rle = NULL;
+        }
+
+        if (this->events[this_event].event->object.overlay != NULL) {
+          free (this->events[this_event].event->object.overlay);
+          this->events[this_event].event->object.overlay = NULL;
         }
-        _x_freep (&this->events[this_event].event->object.overlay);
         break;
 
       default:
@@ -495,156 +507,6 @@ static int video_overlay_event( video_overlay_t *this, int64_t vpts ) {
   return processed;
 }
 
-void _x_overlay_clut_yuv2rgb(vo_overlay_t *overlay, int video_color_matrix)
-{
-  int cm = 10; /* ITU-R 601 (SD) */
-
-  if (!overlay->rgb_clut) {
-    uint8_t *p = (uint8_t *)overlay->color;
-    if ((p[3] == 'X') && (p[7] == 'C') && (p[11] == 'M')) {
-      cm = p[15];
-      if ((cm >> 1) == 2) /* undefined */
-        cm = video_color_matrix;
-    }
-    _x_clut_yuv2rgb(overlay->color, sizeof(overlay->color) / sizeof (overlay->color[0]), cm);
-    overlay->rgb_clut++;
-  }
-
-  if (!overlay->hili_rgb_clut) {
-    _x_clut_yuv2rgb(overlay->hili_color, sizeof (overlay->color) / sizeof (overlay->color[0]), cm);
-    overlay->hili_rgb_clut++;
-  }
-}
-
-static void clut_to_argb(const uint32_t *color, const uint8_t *trans, int num_items, uint32_t *argb, const char *format)
-{
-  int i;
-  union {
-    uint32_t u32;
-    clut_t   c;
-  } tmp ;
-
-  if (!strcmp(format, "BGRA")) {
-    for (i = 0; i < num_items; i++) {
-      tmp.u32 = color[i];
-      uint8_t *rgba = (uint8_t*)(argb + i);
-      rgba[0] = tmp.c.cb;
-      rgba[1] = tmp.c.cr;
-      rgba[2] = tmp.c.y;
-      rgba[3] = trans[i] * 255 / 15;
-    }
-  }
-  else if (!strcmp(format, "RGBA")) {
-    for (i = 0; i < num_items; i++) {
-      tmp.u32 = color[i];
-      uint8_t *rgba = (uint8_t*)(argb + i);
-      rgba[0] = tmp.c.y;
-      rgba[1] = tmp.c.cr;
-      rgba[2] = tmp.c.cb;
-      rgba[3] = trans[i] * 255 / 15;
-    }
-  }
-  else {
-    fprintf(stderr, "clut_to_argb: unknown format %s\n", format);
-  }
-}
-
-#define LUT_SIZE (sizeof(overlay->color)/sizeof(overlay->color[0]))
-void _x_overlay_to_argb32 (const vo_overlay_t *overlay, uint32_t *rgba_buf, int stride, const char *format) {
-  const rle_elem_t *rle = overlay->rle, *rle_end = rle + overlay->num_rle;
-  int lines1, lines2, lines3;
-  int pixels1, pixels2, pixels3;
-  int prest, pad = stride - overlay->width;
-  uint32_t *rgba = rgba_buf, colors[LUT_SIZE * 2], color;
-
-  clut_to_argb (overlay->color, overlay->trans, LUT_SIZE, colors, format);
-
-#define GET_DIM(dest,src,max) dest = src; if (dest < 0) dest = 0; else if (dest > max) dest = max;
-  GET_DIM (lines1, overlay->hili_top, overlay->height);
-  GET_DIM (lines2, overlay->hili_bottom - overlay->hili_top + 1, overlay->height - lines1);
-  lines3 = overlay->height - lines1 - lines2;
-  GET_DIM (pixels1, overlay->hili_left, overlay->width);
-  GET_DIM (pixels2, overlay->hili_right - overlay->hili_left + 1, overlay->width - pixels1);
-  pixels3 = overlay->width - pixels1 - pixels2;
-#undef GET_DIM
-  if ((lines2 > 0) && (pixels2 > 0)) { /* highlight */
-    clut_to_argb (overlay->hili_color, overlay->hili_trans, LUT_SIZE, colors + LUT_SIZE, format);
-  } else {
-    lines1 += lines3;
-    lines2 = lines3 = 0;
-    pixels1 += pixels3;
-    pixels2 = pixels3 = 0;
-  }
-
-#define MAKE_LINE(offs) \
-  while (1) { \
-    int pleft = prest > pixels ? pixels : prest; \
-    pixels -= pleft; \
-    prest -= pleft; \
-    while (pleft > 0) { \
-      *rgba++ = color; \
-      pleft--; \
-    } \
-    if (pixels <= 0) \
-      break; \
-    if (rle >= rle_end) \
-      goto _fill; \
-    color = colors[rle->color + offs]; \
-    prest = rle->len; \
-    rle++; \
-  }
-
-  prest = 0;
-  color = 0;
-  /* top */
-  while (lines1 > 0) {
-    int pixels = overlay->width;
-    MAKE_LINE (0);
-    rgba += pad;
-    lines1--;
-  }
-
-  /* highlight */
-  while (lines2 > 0) {
-    /* left */
-    int pixels = pixels1;
-    MAKE_LINE (0);
-    /* highlighted */
-    if (prest > 0)
-      color = colors[rle[-1].color + LUT_SIZE];
-    pixels = pixels2;
-    MAKE_LINE (LUT_SIZE);
-    /* right */
-    if (prest > 0)
-      color = colors[rle[-1].color];
-    pixels = pixels3;
-    MAKE_LINE (0);
-    rgba += pad;
-    lines2--;
-  }
-
-  /* bottom */
-  while (lines3 > 0) {
-    int pixels = overlay->width;
-    MAKE_LINE (0);
-    rgba += pad;
-    lines3--;
-  }
-
-#undef MAKE_LINE
-
-  return;
-
-  _fill:
-  {
-    int n;
-    n = rgba_buf + stride * overlay->height - rgba;
-    if (n > 0)
-      memset (rgba, 0, n * sizeof (uint32_t));
-  }
-}
-#undef LUT_SIZE
-
 /* This is called from video_out.c
  * must call output->overlay_blend for each active overlay.
  */
@@ -712,10 +574,11 @@ static void video_overlay_dispose(video_overlay_manager_t *this_gen) {
   for (i=0; i < MAX_EVENTS; i++) {
     if (this->events[i].event != NULL) {
       if (this->events[i].event->object.overlay != NULL) {
-        _x_freep (&this->events[i].event->object.overlay->rle);
-        _x_freep (&this->events[i].event->object.overlay);
+        if (this->events[i].event->object.overlay->rle)
+          free (this->events[i].event->object.overlay->rle);
+        free (this->events[i].event->object.overlay);
       }
-      _x_freep (&this->events[i].event);
+      free (this->events[i].event);
     }
   }
 
@@ -732,31 +595,9 @@ static void video_overlay_dispose(video_overlay_manager_t *this_gen) {
 
 video_overlay_manager_t *_x_video_overlay_new_manager (xine_t *xine) {
 
-  int i;
-  video_overlay_t *this = calloc (1, sizeof (*this));
-  if (!this)
-    return NULL;
+  video_overlay_t *this;
 
-#ifndef HAVE_ZERO_SAFE_MEM
-  for (i = 0; i < MAX_EVENTS; i++) {
-    this->events[i].event = NULL;
-    this->events[i].next_event = 0;
-  }
-#endif
-
-  for (i = 0; i < MAX_OBJECTS; i++) {
-    this->objects[i].handle = -1;
-#ifndef HAVE_ZERO_SAFE_MEM
-    this->objects[i].object_type = 0;
-    this->objects[i].pts = 0;
-    this->objects[i].overlay = NULL;
-    this->objects[i].palette = NULL;
-    this->objects[i].palette_type = 0;
-#endif
-  }
-  for (i = 0; i < MAX_SHOWING; i++) {
-    this->showing[i].handle = -1;
-  }
+  this = (video_overlay_t *) calloc(1, sizeof(video_overlay_t));
 
   this->xine                              = xine;
   this->video_overlay.init                = video_overlay_init;
@@ -768,6 +609,5 @@ video_overlay_manager_t *_x_video_overlay_new_manager (xine_t *xine) {
   this->video_overlay.redraw_needed       = video_overlay_redraw_needed;
   this->video_overlay.multiple_overlay_blend = video_overlay_multiple_overlay_blend;
 
-  return &this->video_overlay;
+  return (video_overlay_manager_t *) &this->video_overlay;
 }
-
